* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; ODS LISTING;

***********************************************************************************;
*******       MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR EXAMPLE 3b        *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use FitTest macro;
* FitFewer = Name of ODS InfoCrit table for nested model;
* FitMore  = Name of ODS InfoCrit table for comparison model;
%MACRO FitTest(FitFewer=,FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE9 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE9;
%MEND FitTest;

* To use TotalR2 macro;
* DV =        Case-sensitive name of dependent variable;
* PredFewer = Name of OUTPM= data file of predicted outcomes for nested model;
* PredMore =  Name of OUTPM= data file of predicted outcomes for comparison model;
%MACRO TotalR2(DV=,PredFewer=,PredMore=);
PROC CORR NOPRINT NOSIMPLE DATA=&PredFewer. OUTP=CorrFewer; VAR pred &DV.; RUN;
PROC CORR NOPRINT NOSIMPLE DATA=&PredMore.  OUTP=CorrMore;  VAR pred &DV.; RUN;
DATA CorrFewer; LENGTH Name $30.; SET CorrFewer; Name="&PredFewer."; RUN;
DATA CorrMore;  LENGTH Name $30.; SET CorrMore;  Name="&PredMore.";  RUN;
DATA CorrCompare; LENGTH Name $30.; SET CorrFewer CorrMore; 
     PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; DROP Pred; RUN;
DATA CorrCompare; SET CorrCompare; TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
TITLE9 "Total R2 (% Reduction) for &PredFewer. vs. &PredMore.";
PROC PRINT NOOBS DATA=CorrCompare; RUN; TITLE9; 
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested model;
* CovMore =  Name of ODS CovParms table for comparison model;
%MACRO PseudoR2(NCov=,CovFewer=,CovMore=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore.;  LENGTH Name $30.; SET &CovMore.;  Name="&CovMore.";  RUN;
DATA CovCompare; LENGTH Name $30.; SET &CovFewer. &CovMore.; RUN;
DATA CovCompare; SET CovCompare; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(4,1)","UN(3,2)","UN(4,2)","UN(4,3)") 
     THEN DELETE; RUN;
TITLE9 "PsuedoR2 (% Reduction) for &CovFewer. vs. &CovMore.";
PROC PRINT NOOBS DATA=CovCompare; RUN; TITLE9;
%MEND PseudoR2;


***********************************************************************************;
*******                  BEGIN DATA MANIPULATION FOR EXAMPLE 3b             *******;
*******                  CHANGE "filesave" to your directory                *******;
***********************************************************************************;

* Defining global variable for file location to be replaced in code below;
%LET filesave= C:\Dropbox\18_CLDP945\CLDP945_Example03b;
* Location for SAS files for these models (uses macro variable filesave);
LIBNAME filesave "&filesave.";
    
* Read into work library and center predictors;
DATA work.Example3b; SET filesave.Example3b;
	* Level-2 effect of Negative Mood (mean=0, SD=1);
	  pmnm0 = pmnegmood - 0; LABEL pmnm0 = "PM Negative Mood (0=0)";
	* Level-1 effect to use with PERSON-MEAN-CENTERING;
	  wpnm  = negmood - pmnegmood; LABEL wpnm  = "WP Negative Mood (0=PM)";
	* Level-1 effect to use with GRAND-MEAN-CENTERING;
	  tvnm0 = negmood - 0; LABEL tvnm0  = "TV Negative Mood (0=0)";
	* Gender already exists; 
	  LABEL sexMW = "Sex (0=M, 1=W)"; 
RUN;

******************************************************************************;
*** PART 1: VARIANCE DECOMPOSITION;
******************************************************************************;

* Open output directory to save results to;
ODS RTF FILE="&filesave.\CLDP945_Example03b_Output.rtf" STYLE=HTMLBlue STARTPAGE=NO;

TITLE "Empty Means, Random Intercept Model for Glucose (TV Outcome)";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL lGlucAM = / SOLUTION DDFM=Satterthwaite; 
	RANDOM INTERCEPT  / VCORR SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT CovParms=CovEmpty InfoCrit=FitEmpty; * Save covparms, fit;
RUN;

TITLE "Empty Means, Random Intercept Model for NegativeMood (TV Predictor)";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL negmood = / SOLUTION DDFM=Satterthwaite; 
	RANDOM INTERCEPT  / VCORR SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
RUN;

******************************************************************************;
*** PART 2: PERSON-MEAN-CENTERING OF NEGATIVE MOOD TO PREDICT GLUCOSE;
******************************************************************************;

TITLE "Adding both fixed effects of negative mood under Person-MC";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL lGlucAM = WPnm PMnm0 / SOLUTION DDFM=Satterthwaite OUTPM=PredMood; 
	RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT CovParms=CovMood InfoCrit=FitMood; * Save covparms, fit;
	ESTIMATE "Within-Person Effect" 	WPnm 1;
	ESTIMATE "Between-Person Effect" 	PMnm0 1;
	ESTIMATE "Contextual Effect" 		WPnm -1 PMnm0 1;
RUN; 

* Calculate PseudoR2 relative to empty model;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore=CovMood);

* Calculate Total R2 (nothing to compare it to yet);
PROC CORR DATA=PredMood; VAR lGlucAM pred; RUN;

* Calculate difference in model fit relative to empty model;
%FitTest(FitFewer=FitEmpty, FitMore=FitMood);

TITLE "Add random effect of WP negative mood under Person-MC";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL lGlucAM = WPnm PMnm0 / SOLUTION DDFM=Satterthwaite; 
	RANDOM INTERCEPT WPnm / SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT InfoCrit=FitRandMood; * Save fit;
	ESTIMATE "Within-Person Effect" 	WPnm 1;
	ESTIMATE "Between-Person Effect" 	PMnm0 1;
	ESTIMATE "Contextual Effect" 		WPnm -1 PMnm0 1;
RUN;

* Calculate difference in model fit relative to fixed WPnm model;
%FitTest(FitFewer=FitMood, FitMore=FitRandMood);

TITLE "Add 3 effects of sex (0=M, 1=F) under Person-MC negative mood";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL lGlucAM = WPnm PMnm0 sexMW WPnm*sexMW PMnm0*sexMW 
						/ SOLUTION DDFM=Satterthwaite OUTPM=PredSex; 
	RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT CovParms=CovSex InfoCrit=FitSex; * Save covparms, fit;
	ESTIMATE "Intercept: Men (Mood=0)"			 intercept 1   sexMW 0;
	ESTIMATE "Intercept: Women (Mood=0)"		 intercept 1   sexMW 1;
	ESTIMATE "Intercept: Women Diff (Mood=0)" 	               sexMW 1;
	ESTIMATE "Within-Person Effect: Men" 		 WPnm 1   WPnm*sexMW 0;
	ESTIMATE "Within-Person Effect: Women" 		 WPnm 1   WPnm*sexMW 1;
	ESTIMATE "Within-Person Effect: Women Diff"            WPnm*sexMW 1;
	ESTIMATE "Between-Person Effect: Men" 		 PMnm0 1 PMnm0*sexMW 0;
	ESTIMATE "Between-Person Effect: Women" 	 PMnm0 1 PMnm0*sexMW 1;
	ESTIMATE "Between-Person Effect: Women Diff"          PMnm0*sexMW 1;
	ESTIMATE "Contextual Effect: Men" 			 WPnm -1 WPnm*sexMW  0 PMnm0 1 PMnm0*sexMW 0;
	ESTIMATE "Contextual Effect: Women" 		 WPnm -1 WPnm*sexMW -1 PMnm0 1 PMnm0*sexMW 1;
	ESTIMATE "Contextual Effect: Women Diff"              WPnm*sexMW -1         PMnm0*sexMW 1;
RUN;

* Calculate PseudoR2 relative to fixed-mood-only model;
%PseudoR2(NCov=2, CovFewer=CovMood, CovMore=CovSex);

* Calculate Total R2 change relative to fixed-mood-only model;
%TotalR2(DV=lGlucAM, PredFewer=PredMood, PredMore=PredSex);

* Calculate difference in model fit relative to fixed-mood-only model;
%FitTest(FitFewer=FitMood, FitMore=FitSex);


TITLE "SAS Person-MC Mood and Sex (0=M, 1=W) USING CLASS STATEMENT";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day sexMW;
	MODEL lGlucAM = WPnm PMnm0 sexMW WPnm*sexMW PMnm0*sexMW 
		/ SOLUTION DDFM=Satterthwaite; 
	RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT CovParms=CovSex InfoCrit=FitSex; * Save covparms, fit;
	ESTIMATE "Intercept: Men"			intercept 1 sexMW  1 0;
	ESTIMATE "Intercept: Women"			intercept 1 sexMW  0 1;
	ESTIMATE "Intercept: Women Diff" 		            sexMW -1 1;

	ESTIMATE "Within-Person Effect: Men" 		WPnm 1 WPnm*sexMW  1 0;
	ESTIMATE "Within-Person Effect: Women" 		WPnm 1 WPnm*sexMW  0 1;
	ESTIMATE "Within-Person Effect: Women Diff" 	       WPnm*sexMW -1 1;

	ESTIMATE "Between-Person Effect: Men" 	   	 PMnm0 1 PMnm0*sexMW  1 0;
	ESTIMATE "Between-Person Effect: Women" 	 PMnm0 1 PMnm0*sexMW  0 1;
	ESTIMATE "Between-Person Effect: Women Diff"          PMnm0*sexMW -1 1; 

	ESTIMATE "Context Effect: Men"   WPnm -1 WPnm*sexMW -1  0 PMnm0 1 PMnm0*sexMW  1 0;
	ESTIMATE "Context Effect: Women" WPnm -1 WPnm*sexMW  0 -1 PMnm0 1 PMnm0*sexMW  0 1;
	ESTIMATE "Context Effect: Women Diff"    WPnm*sexMW  1 -1         PMnm0*sexMW -1 1;
RUN;

* Calculate PseudoR2 relative to fixed-mood-only model;
%PseudoR2(NCov=2, CovFewer=CovMood, CovMore=CovSex);

* Calculate Total R2 change relative to fixed-mood-only model;
%TotalR2(DV=lGlucAM, PredFewer=PredMood, PredMore=PredSex);

* Calculate difference in model fit relative to fixed-mood-only model;
%FitTest(FitFewer=FitMood, FitMore=FitSex);


******************************************************************************;
*** PART 3: GRAND-MEAN-CENTERING OF NEGATIVE MOOD TO PREDICT GLUCOSE;
******************************************************************************;

TITLE "Add fixed effect of level-1 negative mood only under Grand-MC";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL lGlucAM = TVnm0 / SOLUTION DDFM=Satterthwaite; 
	RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT CovParms=CovSmush; * Save covparms;
RUN;

* Calculate PseudoR2 relative to empty model;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore=CovSmush);

TITLE "Add fixed effect of level-2 negative mood under Grand-MC";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL lGlucAM = TVnm0 PMnm0 / SOLUTION DDFM=Satterthwaite OUTPM=PredTVMood; 
	RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT CovParms=CovTVmood InfoCrit=FitTVmood; * Save covparms, fit;
	ESTIMATE "Within-Person Effect" 	TVnm0 1;
	ESTIMATE "Between-Person Effect" 	TVnm0 1 PMnm0 1;
	ESTIMATE "Contextual Effect" 		PMnm0 1;
RUN;

* Calculate PseudoR2 relative to smushed model;
%PseudoR2(NCov=2, CovFewer=CovSmush, CovMore=CovTVmood);

TITLE "Add random effect of level-1 negative mood only under Grand-MC";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL lGlucAM = TVnm0 PMnm0 / SOLUTION DDFM=Satterthwaite; 
	RANDOM INTERCEPT TVnm0 / SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT InfoCrit=FitTVRandMood; * Save fit;
	ESTIMATE "Within-Person Effect" 	TVnm0 1;
	ESTIMATE "Between-Person Effect" 	TVnm0 1 PMnm0 1;
	ESTIMATE "Contextual Effect" 		PMnm0 1;
RUN;

* Calculate difference in model fit relative to fixed-mood-only model;
%FitTest(FitFewer=FitTVMood, FitMore=FitTVRandMood);

TITLE "Add sex, sex*TVnm0, and sex*PMnm0 under Grand-MC";
PROC MIXED DATA=work.Example3b COVTEST NOCLPRINT IC NAMELEN=100 METHOD=ML;
	CLASS ID day;
	MODEL lGlucAM = TVnm0 PMnm0 sexMW TVnm0*sexMW PMnm0*sexMW 
					/ SOLUTION DDFM=Satterthwaite OUTPM=PredTVSex; 
	RANDOM INTERCEPT / SUBJECT=ID TYPE=UN; 
	REPEATED day / SUBJECT=ID TYPE=VC; 
	ODS OUTPUT CovParms=CovTVSex InfoCrit=FitTVSex; * Save covparms, fit;
	ESTIMATE "Intercept: Men"				 intercept 1 sexMW 0;
	ESTIMATE "Intercept: Women"				 intercept 1 sexMW 1;
	ESTIMATE "Intercept: Women Diff" 			             sexMW 1;
	ESTIMATE "Within-Person Effect: Men" 		 TVnm0 1 TVnm0*sexMW 0;
	ESTIMATE "Within-Person Effect: Women" 		 TVnm0 1 TVnm0*sexMW 1;
	ESTIMATE "Within-Person Effect: Women Diff"           TVnm0*sexMW 1;
	ESTIMATE "Between-Person Effect: Men" 		 TVnm0 1 TVnm0*sexMW 0 PMnm0 1 PMnm0*sexMW 0;
	ESTIMATE "Between-Person Effect: Women" 	 TVnm0 1 TVnm0*sexMW 1 PMnm0 1 PMnm0*sexMW 1;
	ESTIMATE "Between-Person Effect: Women Diff"          TVnm0*sexMW 1         PMnm0*sexMW 1;
	ESTIMATE "Contextual Effect: Men" 		 	 PMnm0 1 PMnm0*sexMW 0;
	ESTIMATE "Contextual Effect: Women" 	 	 PMnm0 1 PMnm0*sexMW 1;
	ESTIMATE "Contextual Effect: Women Diff" 	         PMnm0*sexMW 1;
RUN;

* Calculate PseudoR2 relative to fixed-mood-only model;
%PseudoR2(NCov=2, CovFewer=CovTVMood, CovMore=CovTVSex);

* Calculate Total R2 change relative to fixed-mood-only model;
%TotalR2(DV=lGlucAM, PredFewer=PredTVMood, PredMore=PredTVSex);

* Calculate difference in model fit relative to fixed-mood-only model;
%FitTest(FitFewer=FitTVMood, FitMore=FitTVSex);

* Close output;
ODS RTF CLOSE;

